package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.CommonConst;
import com.hz.pm.api.common.model.entity.ExcelExportWriter;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.projectlib.helper.ProjectHelper;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectApprovedReq;
import com.hz.pm.api.projectlib.model.req.ProjectIdReq;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.StartProjectDeclareReq;
import com.hz.pm.api.projectlib.model.vo.AnnualPlanListItemVO;
import com.hz.pm.api.projectlib.service.IProjectApplicationService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.todocenter.constant.TodoCenterConst;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.*;

import static com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus.*;

/**
 * <p>
 * AnnualPlanLibManage
 * </p>
 *
 * @author WendyYang
 * @since 14:09 2023/2/11
 */
@Component
@AllArgsConstructor
@Slf4j
public class AnnualPlanLibManage {

    private final IProjectService projectService;
    private final MhUnitCache mhUnitCache;
    private final IProjectApplicationService applicationService;
    private final ProjectStateMachineUtil stateMachine;
    private final UserInfoHelper userInfoHelper;

    /**
     * 年度计划查询状态
     */
    private static final List<ProjectStatus> ANNUAL_PLAN_LIST_STATUS =
            Arrays.asList(ON_ANNUAL_PLAN, TO_BE_DECLARED,
                    PROJECT_APPROVED, TO_BE_PURCHASED, ON_PILOT_RUNNING, ARCHIVED);

    public PageVo<AnnualPlanListItemVO> annulPlanLibList(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(req);
        Integer isTemporaryAugment = req.getIsTemporaryAugment();
        if (Objects.isNull(isTemporaryAugment)) {
            throw new BizException("请传入是否临时增补标志!");
        }
        query.eq(Project::getIsTemporaryAugment, isTemporaryAugment);
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByDesc(Project::getAnnualPlanAddTime);
        query.in(Project::getStatus, CollUtils.fieldList(ANNUAL_PLAN_LIST_STATUS, ProjectStatus::getCode));
        // 数据权限
        if (!buildProjectLibPermission(query)) {
            return PageVo.empty();
        }
        Page<Project> page = projectService.page(req.page(), query);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        PageVo<AnnualPlanListItemVO> result = PageVo.of(new ArrayList<>(), total);
        page.getRecords().forEach(w -> {
            AnnualPlanListItemVO item = new AnnualPlanListItemVO();
            item.setProjectId(w.getId());
            item.setProjectId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectYear(w.getProjectYear());
            item.setProjectType(w.getProjectType());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setBuildOrg(w.getBuildOrgName());
            item.setCreateOn(w.getCreateOn());
            item.setApprovalAmount(w.getApprovalAmount());
            item.setIsStartDeclaredProject(!ON_ANNUAL_PLAN.eq(w.getStatus()));
            result.getRecords().add(item);
        });
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    public void startProjectDeclared(StartProjectDeclareReq req) {
        List<Project> projects = projectService.listByIds(req.getProjectIds());
        projects.forEach(w -> {
            if (!ON_ANNUAL_PLAN.eq(w.getStatus())) {
                throw BizException.wrap("开启方案申报失败");
            }
        });
        projects.forEach(project -> {
            try {
                stateMachine.pass(project);
            } catch (Exception e) {
                throw new BizException("开启方案申报失败");
            }
        });
        projectService.updateBatchById(projects);
    }

    @Transactional(rollbackFor = Exception.class)
    public void projectApproved(ProjectApprovedReq req) {
        Project oldProject = projectService.getById(req.getProjectId());
        Project project = BeanUtil.copyProperties(oldProject, Project.class);
        // 被撤回重新申报的项目,项目ID要置空
        project.setCreateOn(LocalDateTime.now());
        project.setUpdateOn(LocalDateTime.now());
        stateMachine.pass(project);
        //计划出 计划验收时间
        LocalDateTime planAcceptanceTime = getPlanAcceptanceTime(req);
        //立项批复总投资
        project.setApprovalAmount(req.getApprovedAmount());
        //立项批复预算
        project.setApprovalBudget(req.getApprovalBudget());
        project.setApprovedFile(String.valueOf(req.getApprovedFileId()));
        project.setBuildCycle(String.valueOf(req.getBuildCycle()));
        project.setOriginBuildCycle(req.getBuildCycle());
        BizUtils.notNull(req.getBuildPlanFileId(), w -> project.setApprovedConstructionPlanFile(String.valueOf(w)));
        project.setApprovalDate(req.getApprovedDate());
        project.setPlanAcceptanceTime(planAcceptanceTime);
        project.setOriginPlanAcceptanceTime(planAcceptanceTime);
        // 项目编号不变，版本号加1
        project.setProjectCode(oldProject.getProjectCode());
        project.setVersion(oldProject.getVersion() + 1);
        project.setInstCode(TodoCenterConst.Declared.NULL_INST_CODE);
        project.setId(null);
        if (projectService.save(project)) {
            applicationService.saveApplication(project, oldProject, Boolean.TRUE);
            // 将旧的项目版本置为不是最新
            projectService.reverseNewest(project.getProjectCode(), project.getId());
        }
    }

    public LocalDateTime getPlanAcceptanceTime(ProjectApprovedReq req) {
        if (Objects.nonNull(req.getApprovedDate()) && Objects.nonNull(req.getBuildCycle())) {
            return req.getApprovedDate().plusMonths(req.getBuildCycle());
        }
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    public void suspendAnnualPlan(ProjectIdReq req) {
        Project project = projectService.getById(req.getProjectId());
        stateMachine.reject(project);
        projectService.updateById(project);
    }

    public void updateAnnualPlan(ProjectDTO req) {
        Project project = BeanUtil.copyProperties(req, Project.class);
        projectService.updateById(project);
    }

    public void exportList(ProjectListReq param, HttpServletResponse response) {
        param.setPageNumber(CommonConst.EXPORT_PAGE_NUMBER);
        param.setPageSize(CommonConst.EXPORT_PAGE_SIZE);
        Integer isTemporaryAugment = param.getIsTemporaryAugment();
        if (Objects.isNull(isTemporaryAugment)) {
            throw new BizException("请传入是否临时增补标志!");
        }
        param.setIsTemporaryAugment(isTemporaryAugment);
        param.setStatusList(CollUtils.fieldList(ANNUAL_PLAN_LIST_STATUS, ProjectStatus::getCode));
        LambdaQueryWrapper<Project> query = ProjectHelper.projectQuery(param);
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByDesc(Project::getAnnualPlanAddTime);
        List<Project> projects = projectService.list(query);

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();
        String fileName = null;
        if (CommonEnum.NO.getCode().equals(isTemporaryAugment)) {
            fileName = "年度计划库列表";
        } else if (CommonEnum.YES.getCode().equals(isTemporaryAugment)) {
            fileName = "年度计划增补库列表";
        }
        excelExportWriter.setFileName(fileName);
        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getProjectExportDatas(param.getExportOptionList(), projects);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);
    }

    /**
     * build 项目的角色权限到req
     *
     * @param query \
     */
    public boolean buildProjectLibPermission(LambdaQueryWrapper<Project> query) {
        boolean queryState = Boolean.TRUE;
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        Optional<DataScopeDTO> currentUserDataScope = DataScopeUtil.getCurrentUserDataScope(user);
        if (!currentUserDataScope.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = Boolean.FALSE;
        } else {
            switch (currentUserDataScope.get().getRole()) {
                case NORMAL_MEMBER:
                    // 普通用户 只能看到自己单位去申报的
                    query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
                    break;
                case COMPANY_MANAGER:
                    // 单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    List<String> viewUnitIdList = CollUtils.convert(childUnitIds, String::valueOf);
                    query.in(Project::getBuildOrgCode, viewUnitIdList);
                    break;
                case SUPER_ADMIN:
                    // 超级管理员 看到丽水全市的 并且也要判断他 同时是不是单位管理员
                    break;
                default:
                    // 没有权限的话 就让它查不到
                    queryState = Boolean.FALSE;
                    break;
            }
        }
        return queryState;

    }

}
